# -*- coding: utf-8 -*-

"""Library patches."""

import logging
logger = logging.getLogger(__name__)

import sys

from beaker import cache
from beaker.util import coerce_cache_params
from pyramid import mako_templating

def templateLookupFactory(settings):
    """Use the ``settings`` to return a patched ``TemplateLookup`` class."""
    
    class TemplateLookup(mako_templating.PkgResourceTemplateLookup):
        """Sets ``cache_args`` if not passed into the lookup constructor."""
    
        def __init__(self, *args, **kwargs):
            """Patch ``kwargs['cache_args']``."""
            
            if not 'cache_args' in kwargs:
                # Read the ``cache.*`` ones into ``cache_args``.
                cache_args = {}
                prefix = 'mako.cache_args.'
                for key in settings.keys():
                    if key.startswith(prefix):
                        name = key.split(prefix)[1].strip()
                        value = settings[key]
                        try:
                            value = value.strip()
                        except AttributeError:
                            pass
                        cache_args[name] = value
                coerce_cache_params(cache_args)
                cache_args['timeout'] = cache_args.get('expire')
                kwargs['cache_args'] = cache_args
            super(TemplateLookup, self).__init__(*args, **kwargs)
        
    
    return TemplateLookup


def patch_all(settings):
    """Run all the patches."""
    
    # Patch ``mako.lookup.TemplateLookup`` to be our ``TemplateLookup`` class.
    mako_templating.PkgResourceTemplateLookup = templateLookupFactory(settings)




# -*- coding: utf-8 -*-

"""Enforce HTTP Strict Transport Security (see e.g.: `#`_).
  
  _`#`: http://www.imperialviolet.org/2012/07/19/hope9talk.html
"""

import logging
logger = logging.getLogger(__name__)

import urlparse

from pyramid.httpexceptions import HTTPFound, HTTPForbidden
from pyramid.request import Request
from pyramid.settings import asbool

def ensure_secure_url(url, parse_url=None):
    """Add an ``s`` to the ``url`` protocol, if necessary."""
    
    if parse_url is None:
        parse_url = urlparse.urlparse
    
    # If it's not secure, append an ``s`` to the protocol.
    parsed = parse_url(url)
    protocol = parsed.scheme
    if protocol and not protocol.endswith('s'):
        original_protocol = '{0}://'.format(protocol)
        secure_protocol = '{0}s://'.format(protocol)
        url = url.replace(original_protocol, secure_protocol, 1)
    return url


def hsts_redirect_to_https(event):
    """Redirects `http://` GET requests to `https://` and blocks non `https://`
      requests to other request methods.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_request = Mock()
          >>> mock_event = Mock()
          >>> mock_event.request = mock_request
      
      Noop unless ``settings['hsts.force_https']`` is true::
      
          >>> mock_request.registry.settings = {}
          >>> hsts_redirect_to_https(mock_event)
          >>> mock_request.registry.settings = {'hsts.force_https': 'false'}
          >>> hsts_redirect_to_https(mock_event)
          
      Noop if the request is over https://
      
          >>> mock_request.registry.settings = {'hsts.force_https': 'true'}
          >>> mock_request.scheme = 'https'
          >>> hsts_redirect_to_https(mock_event)
      
      Or if the value of the configured ``hsts.protocol_header`` is https::
      
          >>> mock_request.scheme = 'http'
          >>> mock_request.registry.settings = {'hsts.force_https': 'true',
          ...         'hsts.protocol_header': 'Foo'}
          >>> mock_request.headers = {'Foo': 'https'}
          >>> hsts_redirect_to_https(mock_event)
      
      Otherwise if a GET request, redirects to the https equivalent::
      
          >>> mock_request.scheme = 'http'
          >>> mock_request.registry.settings = {'hsts.force_https': 'true',
          ...         'hsts.protocol_header': 'Foo'}
          >>> mock_request.headers = {'Foo': 'http'}
          >>> mock_request.method = 'GET'
          >>> mock_request.url = 'http://foo.com'
          >>> hsts_redirect_to_https(mock_event)
          Traceback (most recent call last):
          ...
          HTTPFound: The resource was found at
      
      Otherwise forbidden::
      
          >>> mock_request.method = 'POST'
          >>> hsts_redirect_to_https(mock_event)
          Traceback (most recent call last):
          ...
          HTTPForbidden: Access was denied to this resource.
      
    """
    
    # Unpack the event.
    request = event.request
    settings = request.registry.settings
    
    # Exit unless told to enforce https.
    should_force_https = asbool(settings.get('hsts.force_https', False))
    if not should_force_https:
        return
    
    # Exit if this is https (or any secure protocol).
    if request.scheme.endswith('s'):
        return
    # E.g.: on Heroku, they pass the protocol though using `X-Forwarded-Proto`.
    protocol_header = settings.get('hsts.protocol_header', None)
    if protocol_header:
        protocol = request.headers.get(protocol_header)
        if protocol and protocol.endswith('s'):
            return
    
    # If this is an insecure GET request, then redirect.
    if request.method == 'GET' or request.method == 'HEAD':
        url_parts = request.url.split('://')
        secure_location = 's://'.join(url_parts)
        raise HTTPFound(location=secure_location)
    
    # Otherwise refuse the request.
    raise HTTPForbidden()

def set_hsts_header(event):
    """Add a ``Strict-Transport-Security`` header to the response.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_request = Mock()
          >>> mock_response = Mock()
          >>> mock_event = Mock()
          >>> mock_event.request = mock_request
          >>> mock_event.response = mock_response
      
      Noop unless ``settings['hsts.force_https']`` is true::
      
          >>> mock_request.registry.settings = {}
          >>> set_hsts_header(mock_event)
          >>> mock_response.headers.add.called
          False
          >>> mock_request.registry.settings = {'hsts.force_https': 'false'}
          >>> set_hsts_header(mock_event)
          >>> mock_response.headers.add.called
          False
      
      Otherwise sets ``Strict-Transport-Security`` header::
      
          >>> mock_request.registry.settings = {'hsts.force_https': 'true'}
          >>> set_hsts_header(mock_event)
          >>> mock_response.headers.add.assert_called_with('Strict-Transport-Security',
          ...         'max-age=8640000 includeSubDomains')
          >>> mock_request.registry.settings = {'hsts.force_https': 'true', 
          ...         'hsts.max_age': 12, 'hsts.include_subdomains': 'false'}
          >>> set_hsts_header(mock_event)
          >>> mock_response.headers.add.assert_called_with('Strict-Transport-Security',
          ...         'max-age=12')
      
    """
    
    # Unpack the event.
    request = event.request
    response = event.response
    settings = request.registry.settings
    
    # If we should force https://
    should_force_https = asbool(settings.get('hsts.force_https', False))
    if should_force_https:
        # Set `Strict-Transport-Security` header to enable hsts.
        max_age = settings.get('hsts.max_age', 8640000)
        include_subdomains = asbool(settings.get('hsts.include_subdomains', True))
        value = 'max-age={0}'.format(max_age)
        if include_subdomains:
            value += ' includeSubDomains'
        response.headers.add('Strict-Transport-Security', value)

def secure_route_url(request, request_cls=None, secure_url=None):
    """Overrides ``route_url`` to make sure the protocol of the resulting
      link is secure.  This makes sure that the ``route_url`` function of
      a secure app running behind an https front end can't mistakenly
      output http links (for example).
    """
    
    # Test jig.
    if request_cls is None:
        request_cls = Request
    if secure_url is None:
        secure_url = ensure_secure_url
    
    # Cache the original ``request.route_url`` method.
    original_route_url = request_cls(request.environ).route_url
    
    def route_url(*args, **kwargs):
        """Get the original ``route_url``.  Force it to use a secure protocol."""
        
        # Get the original url.
        url = original_route_url(*args, **kwargs)
        
        # Return the secured version.
        return secure_url(url)
    
    return route_url




# -*- coding: utf-8 -*-

"""Provide a ``request.track_event(category, action, label=None, value=None)``
  method which fires the equivalent of a client side ``ga._trackEvent(...)``
  request to google analytics.
  
  The request is joined to the transaction and made in a background thread.
"""

__all__ = [
    'get_track_event'
]

import logging
logger = logging.getLogger(__name__)

import threading

from pyga.entities import Event, Session, Visitor
from pyga.requests import Tracker

from .tx import join_to_transaction

def get_track_event(request, join=None, event_cls=None, session_cls=None,
        thread_cls=None, tracker_cls=None, visitor_cls=None):
    """Return the ``joined_flash`` request method."""
    
    # Compose.
    if join is None:
        join = join_to_transaction
    if event_cls is None:
        event_cls = Event
    if session_cls is None:
        session_cls = Session
    if thread_cls is None:
        thread_cls = threading.Thread
    if tracker_cls is None:
        tracker_cls = Tracker
    if visitor_cls is None:
        visitor_cls = Visitor
    
    def track_event(category, action, label=None, value=None, noninteraction=False):
        """Join a call to ping google analytics with the given event in a background thread
          to the current transaction.
        """
        
        # Exit if in development.
        settings = request.registry.settings
        if settings.get('mode', None) == 'development':
            return
        
        # Prepare the tracking id, session id and utm cookies.
        gae_tracking_id = settings['gae.tracking_id']
        if not 'gae_session_id' in request.session:
            request.session['gae_session_id'] = session_cls.generate_session_id()
        gae_session_id = request.session['gae_session_id']
        utma_cookie = request.cookies.get('__utma', None)
        utmb_cookie = request.cookies.get('__utmb', None)
        
        # Instantiate the tracker, event, session and visitor.
        tracker = tracker_cls(gae_tracking_id, request.host)
        event = event_cls(category=category, action=action, label=label,
                value=value, noninteraction=noninteraction)
        session = session_cls()
        session.session_id = gae_session_id
        if utmb_cookie:
            session.extract_from_utmb(utmb_cookie)
        visitor = visitor_cls()
        visitor.extract_from_server_meta(request.environ)
        if utma_cookie:
            visitor.extract_from_utma(utma_cookie)
        
        # Join a background call to track the event to the current transaction.
        track_event = lambda: tracker.track_event(event, session, visitor)
        thread = thread_cls(target=track_event)
        return join(thread.start)
    
    return track_event

